% compute n-order approximation for neoclassical growth model
% based on http://www.econ.duke.edu/~uribe/2nd_order/neoclassical_model.m

clear;

%Define parameters
syms SIG DELTA ALFA BETA RHO;

%Define symbolic variables 
syms c cp k kp a ap;

%Write equations that define the equilibrium
f = [c + kp - (1-DELTA) * k - a * k^ALFA; ...
  c^(-SIG) - BETA * cp^(-SIG) * (ap * ALFA * kp^(ALFA-1) + 1 - DELTA); ...
  log(ap) - RHO * log(a)];
% Define the vector of controls, y, and states, x
x = [k a];
y = c;
xp = [kp ap];
yp = cp;

% Make f a function of the logarithm of the state and control vector
f = subs(f, [x,y,xp,yp], exp([x,y,xp,yp]));

% define variables for steady state values
syms as cs ks;
xs = [ks as];
ys = cs;

% set parameter values
BETA=0.95; %discount rate
DELTA=1; %depreciation rate
ALFA=0.3; %capital share
RHO=0; %persistence of technology shock
SIG=2; %intertemporal elasticity of substitution

% now, write eq conditions as function of state, x, and std dev of shock, s
% let:  y = g(x,s)                 y' = g(h(x,s) + n*e'*s,s)
%       x'= h(x,s) + n*e'*s 
% g() and h() are the unkown policy functions that we want to find
% we will compute Taylor series approximations to them
%  g(x,s) =~ sum G(i1..iN,k) (x(1)-xs(1))^(i1-1) ... s^(k-1)
%  where G(i1 ... iN,k) is the (d^I g)/(dx(1)^(i1-1) .. ds^(k-1))

nX = length(x);
nY = length(y);
% initialize G and H
n = 2;
G = symArray([nY (n+1)*ones(1,nX+1)],'g');
H = symArray([nX (n+1)*ones(1,nX+1)],'h');
H(1:length(xs)) = xs; % steady state x
G(1:length(ys)) = ys; % steady state y

% construct g, h, and g(h)
syms g h gh s e;
[g args cg]= multiTaylor(G,n);
g = subs(g,args,[x-xs,s]);
[h args ch]= multiTaylor(H,n);
h = transpose(h)+[0,s*e];
gh = subs(g,x,h);
gh = subs(gh,args,[x-xs,s]);
h = subs(h,args,[x-xs,s]);
T = [x,g,h,gh];
FT = subs(subs(f),[x,y,xp,yp],T);
eqn = [];
dFT = FT;
for d = 1:n
  dFT = jacobian(dFT,[x,s]);
  eqn = [eqn; reshape(dFT,prod(size(dFT)),1)];
end
for i=1:length(eqn)
  fprintf('working on eqn(%d) ... ',i);
  eqn(i) = subs(eqn(i),[x,s,e],[xs,0,0]);
  fprintf('finished\n');
end
% solve for steady state
fs = subs(f,[x,y,xp,yp],[x,y,x,y]);
[as cs ks] = solve(fs(1),fs(2),fs(3),a,c,k);
as = 0;
cs = sym('log(exp(as)*exp(ks*ALFA) - DELTA*exp(ks))');
cs = subs(cs);
xs = subs([ks as]);
ys = subs(cs);
kss = subs(ks) % steady state capital
css = subs(cs) % steady state consumption

cmd = '';
for i=1:numel(eqn)
   cmd = sprintf('%s,subs(''q=0'',q,eqn(%d))',cmd,i);
end
coeffs = [cg; ch;];% cgh];
unknown = [];
for i=1:numel(coeffs)
    try
        % this will throw an error if coeffs(i) is unknown 
        subs(coeffs(i)); 
    catch 
        % add unknown coeff to list of things we're solving for
        cmd= sprintf('%s,coeffs(%d)',cmd,i);
        unknown = [unknown; coeffs(i)];
    end
end
cmd = ['soln=solve(' cmd(2:length(cmd)) ');'];
eqn = vpa(subs(subs(eqn)),16); % solve will take a very long time with exact eqn
eval(cmd);

f = fields(soln);
if (numel(soln.(f{1}))>1) 
    warning('multiple solutions found\n');
    smat = zeros(length(f),numel(soln.(f{1})));
   for i=1:length(f)
     try
       smat(i,:) = subs(soln.(f{i}))';
     end
   end
end

for i=1:length(f)
    try
      fprintf('%s = %s\n',f{i},char(vpa(soln.(f{i}),4)));
    end
    %eval(sprintf('G(find(G==''%s'')) = subs(soln.(%s));',f{i},f{i}));
end
